import numpy as np

class Config:
    # Random Seed
    RANDOM_STATE = 42
    
    # Missing Values
    MISSING_VALUES = [float('nan'), None, np.nan]
    
    # Special Values that should be treated as separate bins (e.g., -999, -1)
    SPECIAL_VALUES = []
    
    # Binning Defaults
    MAX_BINS = 10
    MIN_BIN_PCT = 0.05
    
    # Selection Defaults
    IV_THRESHOLD = 0.02
    PSI_THRESHOLD = 0.1
    CORR_THRESHOLD = 0.7
    
    # Modeling Defaults
    TEST_SIZE = 0.3
    
import numpy as np
